"""
Stronk module
"""

import secrets as random
import os


LETTERS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
SYMBOLS = '!@#$%+=?&*()'


def generate_random_keys(number_of_keys=1, key_length=16, identify=False):
    """ Returns the generated keys """
    number_of_keys_iterator = 0
    keys = []
    error_message = ''
    if number_of_keys > 100:
        error_message += "ERROR: Number of keys must be <= 100\n"
    if number_of_keys <= 0:
        error_message += "ERROR: Number of keys must be non-zero and non-negative\n"
    if key_length > 256:
        error_message += "ERROR: Key length must be <= 256\n"
    if key_length < 16:
        error_message += "ERROR: Key length must be non-zero and non-negative\n"
    if not error_message:
        while number_of_keys_iterator < number_of_keys:
            key_length_iterator = 0
            key = ''
            while key_length_iterator < key_length:
                key += str(_generate_random_char())
                key_length_iterator += 1
            keys.append(_shuffle_key(key))
            number_of_keys_iterator += 1
        _prettify_output(keys, identify)
        _print_to_txt_file(keys, identify)
        return keys
    _prettify_output(error_message, False)
    return error_message


def generate_sha5_hash():
    """ Generate sha5 hash """
    return 0


def _shuffle_key(key):
    """ Returns the shuffled key """
    random.SystemRandom().shuffle(list(key))
    return ''.join(key)


def _generate_random_char():
    """ Generate a random integer, letter, or symbol """
    character_type = random.randbelow(13)
    if character_type == (0 or 1 or 2):
        return random.randbelow(10)
    if character_type == (3 or 4):
        return random.choice(SYMBOLS)
    return random.choice(LETTERS)


def _prettify_output(output, identify):
    if isinstance(output, list):
        for iterator, out in enumerate(output):
            if identify:
                print("(" + str(iterator + 1) + ") " + out)
            else:
                print(out)
    else:
        print(output)


def _print_to_txt_file(keys, identify):
    current_working_directory = os.getcwd()
    output_file = open(current_working_directory + "/stronk.txt", "w")
    for iterator, key in enumerate(keys):
        if identify:
            output_file.write("(" + str(iterator + 1) + ") " + key + "\n")
        else:
            output_file.write(key + "\n")

    output_file.close()



import unittest
import stronk

from stronk import stronk
from unittest.mock import MagicMock


class stronkTest(unittest.TestCase):
    """ Test suite for the stronk module """

    def setUp(self):
        self.key = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"
        self.mock = MagicMock()
        self.mock[0] = stronk._shuffle_key(self.key)

    def test_shuffle_key_returns_shuffled_key(self):
        self.assertNotEqual(self.mock[0], self.key)

    def test_default_key_generation_is_one_key(self):
        self.assertEqual(len(stronk.generate_random_keys()), 1)

    def test_default_key_generation_key_length_is_16(self):
        self.assertEqual(len(stronk.generate_random_keys()[0]), 16)

    def test_generate_max_100_keys(self):
        self.assertEqual(len(stronk.generate_random_keys(100)), 100)

    def test_error_returns_correct_message_if_keys_greater_than_100(self):
        self.assertEqual(stronk.generate_random_keys(
            101), "ERROR: Number of keys must be <= 100\n")

    def test_error_returns_correct_message_if_keys_is_zero(self):
        self.assertEqual(stronk.generate_random_keys(
            0), "ERROR: Number of keys must be non-zero and non-negative\n")

    def test_error_returns_correct_message_if_keys_is_negative(self):
        self.assertEqual(stronk.generate_random_keys(
            -100), "ERROR: Number of keys must be non-zero and non-negative\n")

    def test_error_returns_correct_message_if_key_length_greater_than_256(self):
        self.assertEqual(stronk.generate_random_keys(
            1, 257), "ERROR: Key length must be <= 256\n")

    def test_error_returns_correct_message_if_key_length_is_positive_and_less_than_16(self):
        self.assertEqual(stronk.generate_random_keys(
            1, 15), "ERROR: Key length must be non-zero and non-negative\n")

    def test_error_returns_correct_message_if_key_length_is_zero(self):
        self.assertEqual(stronk.generate_random_keys(
            1, 0), "ERROR: Key length must be non-zero and non-negative\n")

    def test_error_returns_correct_message_if_key_length_is_negative(self):
        self.assertEqual(stronk.generate_random_keys(
            1, -200), "ERROR: Key length must be non-zero and non-negative\n")

    def test_all_nonzero_and_nonnegative_errors_can_concatenate(self):
        self.assertEqual(stronk.generate_random_keys(
            0, 0), "ERROR: Number of keys must be non-zero and non-negative\nERROR: Key length must be non-zero and non-negative\n")

    def test_all_too_large_errors_can_concatenate(self):
        self.assertEqual(stronk.generate_random_keys(101, 257),
                         "ERROR: Number of keys must be <= 100\nERROR: Key length must be <= 256\n")



import pathlib
from setuptools import find_packages, setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='stronk',
    version='1.1.3',
    description='Strong key generator',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/brianrthompson/stronk',
    author='Brian Thompson',
    author_email='brianrt23@gmail.com',
    license='MIT',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6"
    ],
    packages=find_packages(exclude=("test",)),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "stronk=stronk.__main__:main",
        ]
    },
)



# __main__.py
import sys
import argparse

from stronk import stronk, __version__

def getArguments():
    """ Parses CLI arguments """
    parser = argparse.ArgumentParser(description='Generate strong keys.')
    parser.add_argument('keyAmount', metavar='K', type=int, nargs='?', const='1', default='1',
                        help='number of keys to generate')
    parser.add_argument('keyLength', metavar='L', type=int, nargs='?', const='16', default='16',
                        help='number of keys to generate')
    parser.add_argument('--identify', '-i', action='store_true', required=False,
                        help='print the keys with a number beside them')
    parser.add_argument('--version', '-V', action='version', version='{version}'.format(version=__version__))
    return parser


def main():
    """ Spit out a string regardless of input """
    args = getArguments().parse_args()
    if len(sys.argv) == 2:
        stronk.generate_random_keys(
            args.keyAmount, args.keyLength)
    elif len(sys.argv) >= 3:
        stronk.generate_random_keys(
            args.keyAmount, args.keyLength, args.identify)
    else:
        stronk.generate_random_keys()


if __name__ == "__main__":
    main()
